// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

import Qt 4.7
import mapplugin 1.0

Rectangle {
    id: editor
    width: 800
    height: 600
    state: "editObjects"

    property int blockSize: 30
    property int recentCount: 3
    property int smallMargin : 2
    property int mediumMargin : 5
    property int largeMargin : 20
    property string uiBackgroundColor : "lightGray"
    property string uiHighlightColor : "darkOrange"
    property string uiBorderColor : "black"


    Map {
        id: map
        anchors.fill: parent
        interactive: editor.state !== "draw"
    }

    // help message
    Rectangle {
        id: helpHighlighter
        anchors.bottom: editor.bottom
        anchors.left: editor.left
        width: helpText.width + mediumMargin
        height: helpText.height + mediumMargin
        color: uiBackgroundColor
        opacity: 0.5
        visible: helpText.text !== ""
    }
    Text {
        id: helpText
        anchors.centerIn: helpHighlighter
        text: {
            if (editor.state === "draw")
                "left: inside, right: outside, mid: bridge";
            else if (editor.state === "editObjects")
                "left: select tile, mid: clear, right: place most recently used object";
            else
                "";
        }
    }

    // extend buttons
    TextButton {
        opacity: map.contentX <= blockSize ? 0.7 : 0
        Behavior on opacity { NumberAnimation {} }
        text: "extend"
        anchors.verticalCenter: map.verticalCenter
        anchors.left: map.left
        anchors.margins: -(width - height)/2
        rotation: -90
        onClicked: map.model.extendLeft();
    }
    TextButton {
        opacity: map.contentY <= blockSize ? 0.7 : 0
        Behavior on opacity { NumberAnimation {} }
        text: "extend"
        anchors.horizontalCenter: map.horizontalCenter
        onClicked: map.model.extendUp();
    }
    TextButton {
        opacity: map.contentY >= map.contentHeight - map.height - blockSize ? 0.7 : 0
        Behavior on opacity { NumberAnimation {} }
        text: "extend"
        anchors.horizontalCenter: map.horizontalCenter
        anchors.bottom: map.bottom
        onClicked: {
            map.model.extendDown();
            map.contentY = map.contentHeight - map.height;
        }

    }
    TextButton {
        opacity: map.contentX >= map.contentWidth - map.width - blockSize ? 0.7 : 0
        Behavior on opacity { NumberAnimation {} }
        text: "extend"
        anchors.verticalCenter: map.verticalCenter
        anchors.right: map.right
        anchors.margins: -(width - height)/2
        rotation: 90
        onClicked: {
            map.model.extendRight();
            map.contentX = map.contentWidth - map.width;
        }
    }

    ZoomSlider {
        id: zoomSlider
        anchors.right: editor.right
        anchors.top: editor.top
        anchors.margins: mediumMargin

        value: (editor.blockSize - 15)/15;
        onValueChanged: {
            var prev = editor.blockSize;
            editor.blockSize = 15 + 15*value;
            map.contentX = Math.min(Math.max((map.contentX + map.width/2) * editor.blockSize/prev - map.width/2, 0), map.contentWidth - map.width);
            map.contentY = Math.min(Math.max((map.contentY + map.height/2) * editor.blockSize/prev - map.height/2, 0), map.contentHeight - map.height);
        }
    }

    // edit mode
    Row {
        id: editModeControls
        anchors.right: zoomSlider.left
        anchors.top: editor.top
        anchors.margins: mediumMargin
        spacing: smallMargin

        TextButton {
            Rectangle {
                anchors.fill: parent
                color: "transparent"
                border.width: 3
                border.color: uiHighlightColor
                visible: editor.state === "draw"
            }

            text: "Draw play area"

            onClicked: {
                editor.state = "draw";
            }
        }
        TextButton {
            Rectangle {
                anchors.fill: parent
                color: "transparent"
                border.width: 3
                border.color: uiHighlightColor
                visible: editor.state === "editObjects"
            }

            text: "Edit objects"

            onClicked: {
                editor.state = "editObjects";
            }
        }
    }

    // new/load/save buttons
    Row {
        anchors.left: editor.left
        anchors.top: editor.top
        anchors.margins: mediumMargin
        spacing: smallMargin

        FileDialog { id: fileDialog }

        TextButton {
            text: "New"
            onClicked: reallyNew.visible = true;
            TextButton {
                id: reallyNew
                text: "Do it!"
                visible: false

                anchors.top: parent.bottom
                anchors.left: parent.left

                onClicked: {
                    map.file = "";
                    map.model.newMap(100, 30);
                    map.contentX = 0;
                    map.contentY = 0;
                    visible = false;
                }
                onExited: visible = false;
            }
        }
        TextButton {
            text: "Open..."

            onClicked: {
                var filename = fileDialog.getOpenFileName();
                if (filename === "")
                    return;
                map.file = filename;
                map.contentX = 0;
                map.contentY = 0;
            }
        }
        TextButton {
            text: "Save"
            visible: map.file !== ""

            onClicked: {
                map.model.save(map.file);
            }
        }
        TextButton {
            text: "Save as..."

            onClicked: {
                var saveName = fileDialog.getSaveFileName();
                if (saveName === "")
                    return;
                map.model.save(saveName);
                map.file = saveName;
            }
        }
    }

    TypeSelector {
        id: typeSelector
        anchors.top: editModeControls.bottom
        anchors.bottom: parent.bottom
        anchors.right: zoomSlider.left
        anchors.margins: largeMargin
        opacity: editor.state === "chooseType" ? 1 : 0
        Behavior on opacity { NumberAnimation { duration: 150 } }
    }

    states: [
        State {
            name: "draw"
            PropertyChanges {
                target: map.highlight
                index: undefined
            }
        }
    ]
}
